using System;
using System.Collections.Generic;
using System.Text;
using SuperMap.Data;
using SuperMap.Desktop;
using SuperMap.UI;
using SuperMap.Mapping;
using System.Windows.Forms;


namespace SuperMap.Desktop.ObjectEditing
{
    class _CtrlActionExtend:CtrlAction
    {
        int baseID = 0;
        int modifyID = 0;
        int selCount=0;
        string baseDs;
        string modifyDs ;
        string baseLyrName;
        string modifyLyrName;        

        Boolean m_onRun;

        //Reference line
        GeoLine baseLine;
        //Operation line you want to extend
        GeoLine modifyLine;

        public _CtrlActionExtend(IBaseItem caller)
            : base(caller)
        { 
        }

        
        public override void Run()
        {
            try
            {                
                if (!m_onRun)
                {
                    m_onRun = true;
                    IFormMap formMap = SuperMap.Desktop.Application.ActiveForm as IFormMap;
                    MapControl mapControl = formMap.MapControl;
                    mapControl.GeometrySelected += new GeometrySelectedEventHandler(mapControl_GeometrySelected);
                    mapControl.ActionChanged += new ActionChangedEventHandler(mapControl_ActionChanged);
                }
                else
                {
                    UnRegister();
                }
            }
            catch (Exception ex)
            {
                Application.ActiveApplication.Output.Output(ex.Message);
            }
        }

        void mapControl_ActionChanged(object sender, ActionChangedEventArgs e)
        {
            UnRegister();
        }

        private void UnRegister()
        {
            m_onRun = false;
            IFormMap formMap = SuperMap.Desktop.Application.ActiveForm as IFormMap;
            MapControl mapControl = formMap.MapControl;
            mapControl.GeometrySelected -= new GeometrySelectedEventHandler(mapControl_GeometrySelected);
            mapControl.ActionChanged -= new ActionChangedEventHandler(mapControl_ActionChanged);
        }
        /// <summary>
        /// First select the reference line, then the operation line. The process of extending undershoot is as follows:
        /// 1.Get the intersection of the two liens.
        /// 2.If the intersection is not on the operation line,extend it to the intersection.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void mapControl_GeometrySelected(object sender, GeometrySelectedEventArgs e)
        {
            if (selCount == 0)
            {              
                //Click to get the reference line
                IFormMap formMap = Application.ActiveForm as IFormMap;
                MapControl mapControl = formMap.MapControl;
                Selection[] objSels = mapControl.Map.FindSelection(true);
                Selection objSel = objSels[0];
                if (objSel.Dataset.Type != DatasetType.Line && objSel.Count !=0)
                {
                    Application.ActiveApplication.Output.Output("Please select the reference line");
                    return;
                }

                Recordset objRec = objSel.ToRecordset();

                baseLyrName = objSel.Dataset.Name;
                baseDs = objSel.Dataset.Datasource.Alias;
                baseID = objRec.GetID();
                baseLine = objRec.GetGeometry() as GeoLine;

                objRec.Close();
                objRec.Dispose();

                selCount += 1;
            }
            else if (selCount == 1)
            {
                //Click to get the operation line
                IFormMap formMap = Application.ActiveForm as IFormMap;
                MapControl mapControl = formMap.MapControl;
                Selection[] objSels = mapControl.Map.FindSelection(true);
                Selection objSel = objSels[0];

                if (objSel.Dataset.Type != DatasetType.Line && objSel.Count !=1)
                {
                    Application.ActiveApplication.Output.Output("Please select the operation line");
                    return;
                }

                Recordset objRec = objSel.ToRecordset();
                modifyLyrName = objSel.Dataset.Name;
                modifyDs = objSel.Dataset.Datasource.Alias;
                modifyID = objRec.GetID();

                modifyLine = objRec.GetGeometry() as GeoLine;

                objRec.Close();
                objRec.Dispose();

                selCount += 1;
                
            }

            if (modifyID != 0 && baseID != 0)
            {
                //Get the intersection of the reference line and the operation line       
                Point2Ds basePnts = baseLine[0];
                Point2Ds modifyPnts = modifyLine[0];
                Point2D intersectPnt = Geometrist.IntersectLine(basePnts[0], basePnts[1], modifyPnts[0], modifyPnts[1], true);

                //If the intersection is not on the operation line, extend it to the reference line
                Boolean bOnBaseLine = Geometrist.IsPointOnLine(intersectPnt, basePnts[0], basePnts[1], false);
                Boolean bOnModifyLine = Geometrist.IsPointOnLine(intersectPnt, modifyPnts[0], modifyPnts[1], false);
                if (bOnBaseLine == true && bOnModifyLine == false)
                {
                    modifyPnts.Add(intersectPnt);
                    modifyLine[0] = modifyPnts;

                    IFormMap formMap = Application.ActiveForm as IFormMap;
                    MapControl mapControl = formMap.MapControl;
                    Workspace workspace = mapControl.Map.Workspace;
                    Datasource objDs = workspace.Datasources[modifyDs];
                    DatasetVector objDtv = (DatasetVector)objDs.Datasets[modifyLyrName];

                    //Extend the operation line
                    Recordset objRec = objDtv.Query("SmID=" + modifyID, CursorType.Dynamic);

                    objRec.MoveFirst();
                    objRec.Edit();
                    objRec.SetGeometry(modifyLine);
                    objRec.Update();
                    objRec.Close();
                    objRec.Dispose();
                    mapControl.Map.Refresh();
                    baseLine.Dispose();
                    modifyLine.Dispose();
                }

                selCount = 0;
                baseID = 0;
                baseDs = "";
                baseLyrName = "";
                modifyID = 0;
                modifyDs = "";
                modifyLyrName = "";

                UnRegister();
            }
            
        }

        public override CheckState Check()
        {
            CheckState result = CheckState.Unchecked;
            if (m_onRun)
            {
                result = CheckState.Checked;
            }
            return result;
        }

        public override bool Enable()
        {
           // IBaseItem item = Application.ActiveApplication.MainForm.RibbonManager[typeof(_CtrlActionTrim)];
            return true;// item.CtrlAction.Check() == CheckState.Unchecked;
        }
    }
}
